@@ -1,6 +1,6 @@
 /* 
    cadaver, command-line DAV client
-   Copyright (C) 1999-2007, Joe Orton <joe@manyfish.co.uk>, 
+   Copyright (C) 1999-2024, Joe Orton <joe@manyfish.co.uk>,
    except where otherwise indicated.
                                                                      
    This program is free software; you can redistribute it and/or modify
@@ -43,6 +43,9 @@
 #include <locale.h>
 #endif
 
+#ifdef HAVE_LANGINFO_CODESET
+#include <langinfo.h>
+#endif
 
 #include <errno.h>
 
@@ -82,20 +85,16 @@
 static netrc_entry *netrc_list; /* list of netrc entries */
 #endif
 
-/* Module-private state: */
+/* Global state: */
 const char *lock_store_fn = NULL;
 static char *progname; /* argv[0] */
 static char *rcfile;
-
-/* Global state: */
-char *proxy_hostname;
-int proxy_port;
-char *server_username = NULL, *server_password = NULL;
+static char *proxy_hostname;
+static char *server_username, *server_password;
 
 /* Current session state. */
 struct session session;
-
-ne_ssl_client_cert *client_cert;
+static ne_ssl_client_cert *client_cert;
 
 int tolerant; /* tolerate DAV-enabledness failure */
 
@@ -110,7 +109,7 @@ static enum out_state {
     out_transfer_done /* a complete transfer */
 } out_state;   
 
-/* Protoypes */
+/* Prototypes */
 
 static void quit_handler(int signo);
 
@@ -125,8 +124,8 @@ static int supply_creds_proxy(void *userdata, const char *realm, int attempt,
 static void usage(void)
 {
     printf(_(
-"Usage: %s [OPTIONS] http://hostname[:port]/path\n"
-"  Port defaults to 80, path defaults to '/'\n"
+"Usage: %s [OPTIONS] URL\n"
+"  URL must be an absolute URI using the http: or https: scheme.\n"
 "Options:\n"
 "  -t, --tolerant            Allow cd/open into non-WebDAV enabled collection.\n"
 "  -r, --rcfile=FILE         Read script from FILE instead of ~/.cadaverrc.\n"
@@ -150,10 +149,11 @@ static void finish_locking(void)
 
 void close_connection(void)
 {
-    ne_session_destroy(session.sess);
-    session.connected = 0;
-    printf(_("Connection to `%s' closed.\n"),
-           session.uri.host);
+    if (session.sess) ne_session_destroy(session.sess);
+    session.sess = NULL;
+    if (session.connected && session.uri.host)
+        printf(_("Connection to `%s' closed.\n"), session.uri.host);
+    session.connected = false;
     ne_uri_free(&session.uri);
     if (session.lastwp)
         ne_free(session.lastwp);
@@ -194,6 +194,14 @@ static int cert_verify(void *ud, int failures, const ne_ssl_certificate *c)
     else
         puts(_("WARNING: Untrusted server certificate presented:\n"));
 
+#if NE_VERSION_MAJOR > 0 || NE_VERSION_MINOR > 31
+    tmp = ne_ssl_cert_hdigest(c, NE_HASH_SHA256|NE_HASH_COLON);
+    if (tmp) {
+        printf(_("Server certificate SHA-256 digest: %s\n"), tmp);
+        ne_free(tmp);
+    }
+#endif
+
     if (failures & NE_SSL_IDMISMATCH) {
 	printf(_("Certificate was issued to hostname `%s' rather than `%s'\n"),
 	       ne_ssl_cert_identity(c), session.uri.host);
@@ -261,109 +269,77 @@ static void provide_clicert(void *userdata, ne_session *sess,
     
 }
 
-static int setup_ssl(void)
+static void setup_ssl(ne_session *sess)
 {
     char *ccfn = get_option(opt_clicert);
 
-    ne_ssl_trust_default_ca(session.sess);
+    ne_ssl_trust_default_ca(sess);
 	      
-    ne_ssl_set_verify(session.sess, cert_verify, NULL);
+    ne_ssl_set_verify(sess, cert_verify, NULL);
 
     if (ccfn) {
         client_cert = ne_ssl_clicert_read(ccfn);
         if (client_cert) {
-            ne_ssl_provide_clicert(session.sess, provide_clicert, ccfn);
-        } else {
-            printf("Could not load client certificate from `%s'.\n",
-                   ccfn);
+            ne_ssl_provide_clicert(sess, provide_clicert, ccfn);
+        }
+        else {
+            printf(_("Could not load client certificate from `%s'.\n"), ccfn);
         }
     }
-
-    return 0;
 }
 
-/* FIXME: Leaky as a bucket */
 void open_connection(const char *url)
 {
-    char *proxy_host = get_option(opt_proxy), *pnt;
+    const char *proxy_host = get_option(opt_proxy);
     ne_server_capabilities caps;
-    int ret, use_ssl = 0;
-    ne_session *sess;
+    int ret;
+    ne_session *sess = NULL;
+    unsigned int proxy_port = 8080;
 
-    if (session.connected) {
-	close_connection();
-    } else {
-        ne_uri_free(&session.uri);
-        if (session.lastwp) {
-            ne_free(session.lastwp);
-            session.lastwp = NULL;
+    close_connection();
+
+    /* Parse the URL */
+    if (ne_uri_parse(url, &session.uri) || session.uri.path == NULL
+        || session.uri.scheme == NULL || session.uri.host  == NULL) {
+        printf(_("Could not parse URL `%s'\n"), url);
+        goto fail;
+    }
+
+    if (!session.uri.port)
+        session.uri.port = ne_uri_defaultport(session.uri.scheme);
+
+    /* Collections must be slash-terminated to avoid a redirect
+     * round-trip. */
+    if (!ne_path_has_trailing_slash(session.uri.path)) {
+        char *pnt = ne_concat(session.uri.path, "/", NULL);
+        ne_free(session.uri.path);
+        session.uri.path = pnt;
+    }
+
+    sess = ne_session_create(session.uri.scheme, session.uri.host, session.uri.port);
+    session.sess = sess;
+
+    if (ne_strcasecmp(session.uri.scheme, "https") == 0) {
+        if (!ne_has_support(NE_FEATURE_SSL)) {
+            printf(_("No SSL/TLS support, cannot use URL `%s'\n"), url);
+            goto fail;
         }
+        setup_ssl(sess);
     }
-
-    /* Single argument: see whether we have a path or scheme */
-    if (strchr(url, '/') == NULL) {
-	/* No path, no scheme -> just a hostname */
-	pnt = strchr(url, ':');
-	if (pnt != NULL) {
-	    *pnt++ = '\0';
-	    session.uri.port = atoi(pnt);
-	} else {
-	    session.uri.port = 80;
-	}
-	session.uri.host = ne_strdup(url);
-	session.uri.scheme = ne_strdup("http");
-    } else {
-	/* Parse the URL */
-	if (ne_uri_parse(url, &session.uri) || session.uri.host == NULL) {
-	    printf(_("Could not parse URL `%s'\n"), url);
-	    return;
-	}
-
-	if (session.uri.scheme == NULL)
-	    session.uri.scheme = ne_strdup("http");
-
-	if (!session.uri.port)
-	    session.uri.port = ne_uri_defaultport(session.uri.scheme);
-
-	if (strcasecmp(session.uri.scheme, "https") == 0) {
-	    if (!ne_has_support(NE_FEATURE_SSL)) {
-		printf(_("SSL is not enabled.\n"));
-		return;
-	    }
-	    use_ssl = 1;
-	}
-    }
-
-    session.sess = ne_session_create(session.uri.scheme, session.uri.host,
-                                     session.uri.port);
-    sess = session.sess;
-    
-    if (use_ssl && setup_ssl()) {
-	return;
+    else if (ne_strcasecmp(session.uri.scheme, "http")) {
+        printf(_("URL scheme '%s' not supported.\n"), session.uri.scheme);
+        goto fail;
     }
 
     ne_lockstore_register(session.locks, sess);
     ne_redirect_register(sess);
     ne_set_notifier(sess, notifier, NULL);
-
-    if (session.uri.path == NULL) {
-	session.uri.path = ne_strdup("/");
-    } else {
-	if (!ne_path_has_trailing_slash(session.uri.path)) {
-	    pnt = ne_concat(session.uri.path, "/", NULL);
-	    free(session.uri.path);
-	    session.uri.path = pnt;
-	}
-    }
+    ne_set_session_flag(sess, NE_SESSFLAG_PERSIST, get_bool_option(opt_keepalive));
+    ne_set_session_flag(sess, NE_SESSFLAG_EXPECT100, get_bool_option(opt_expect100));
 
     /* Get the proxy details */
-    if (proxy_host != NULL) {
-	if (get_option(opt_proxy_port) != NULL) {
-	    proxy_port = atoi(get_option(opt_proxy_port));
-	} else {
-	    proxy_port = 8080;
-	}
-	proxy_hostname = proxy_host;
+    if (proxy_host && get_option(opt_proxy_port) != NULL) {
+        proxy_port = atoi(get_option(opt_proxy_port));
     }
 
 #ifdef ENABLE_NETRC
@@ -378,28 +354,30 @@ void open_connection(const char *url)
 	}
     }
 #endif /* ENABLE_NETRC */
-#ifdef NE_SESSFLAG_EXPECT100
-	ne_set_session_flag(session.sess, NE_SESSFLAG_EXPECT100, get_bool_option(opt_expect100));
-#endif /* NE_SESSFLAG_EXPECT100 */
     session.connected = 0;
 
     ne_set_useragent(session.sess, "cadaver/" PACKAGE_VERSION);
     ne_set_server_auth(session.sess, supply_creds_server, NULL);
     ne_set_proxy_auth(session.sess, supply_creds_proxy, NULL);
-    
-    if (proxy_host) {
-	ne_session_proxy(session.sess, proxy_hostname, proxy_port);
+
+    if (get_bool_option(opt_systemproxy)) {
+        ne_session_system_proxy(session.sess, 0);
+    }
+    else if (proxy_host) {
+        if (proxy_hostname) ne_free(proxy_hostname);
+        proxy_hostname = ne_strdup(proxy_host);
+        ne_session_proxy(session.sess, proxy_host, proxy_port);
     }
 
     ret = ne_options(session.sess, session.uri.path, &caps);
     
     switch (ret) {
     case NE_OK:
-	session.connected = true;
-	if (set_path(session.uri.path)) {
-	    close_connection();
-	}
-	break;
+	if (set_path(session.uri.path) == 0) {
+            session.connected = true;
+            return;
+        }
+        break;
     case NE_CONNECT:
 	if (proxy_host) {
 	    printf(_("Could not connect to `%s' on port %d:\n%s\n"),
@@ -418,6 +396,8 @@ void open_connection(const char *url)
 	break;
     }
 
+fail:
+    close_connection();
 }
        
 /* Sets proxy server from hostport argument */    
@@ -428,9 +408,10 @@ static void set_proxy(const char *str)
     pnt = strchr(hostname, ':');
     if (pnt != NULL) {
 	*pnt++ = '\0';
+        if (*pnt)
+            set_option(opt_proxy_port, ne_strdup(pnt));
     }
     set_option(opt_proxy, (void *)hostname);
-    set_option(opt_proxy_port, pnt);
 }
 
 static void parse_args(int argc, char **argv)
@@ -478,12 +459,13 @@ static char *read_command(void)
     char prompt[BUFSIZ];
 
     if (session.uri.path) {
-        char *p = ne_path_unescape(session.uri.path);
-	ne_snprintf(prompt, BUFSIZ, "dav:%s%c ", p,
+        char *p = native_path_from_uri(session.uri.path);
+        ne_snprintf(prompt, BUFSIZ, "dav:%s%c ", p,
                     session.isdav ? '>' : '?');
         ne_free(p);
-    } else {
-	sprintf(prompt, "dav:!> ");
+    }
+    else {
+        ne_strnzcpy(prompt, "dav:!> ", sizeof prompt);
     }
 
     return readline(prompt); 
@@ -641,14 +623,16 @@ static int init_rcfile(void)
 #define rl_filename_completion_function filename_completion_function
 #endif
 
+/* TODO:
+ * 1. "ls /dav/foo/<comp>" should complete within that path not CWD
+ * 2. quoting should happen
+ */
 static char *remote_completion(const char *text, int state)
 {
     static struct resource *reslist, *current;
-    static int len;
+    static size_t len;
     static time_t last_fetch;
     static char *last_path;
-
-    char *name;
     
     if (state == 0) {
 	/* Check to see if we should refresh the dumb cache.
@@ -682,39 +666,39 @@ static char *remote_completion(const char *text, int state)
 	    ne_set_notifier(session.sess, notifier, NULL);
 
 	    last_path = ne_strdup(session.uri.path);
-	}
-
+        }
 	current = reslist;
-	len = strlen(text);
+        len = strlen(text);
 	time(&last_fetch);
     }
 
     while (current) {
-	/* Massage the absolute URI to a URI relative to our path */
-	/* Copy & paste & search & replace from ls.c */
-	if (ne_path_has_trailing_slash(current->uri)) {
-	    current->uri[strlen(current->uri)-1] = '\0';
-	}
+        const char *path = current->uri;
+        char *native_path;
 
-	name = strrchr(current->uri, '/');
-	if (name != NULL && strlen(name+1) > 0) {
-	    name++;
-	} else {
-	    name = current->uri;
-	}
-	name = ne_path_unescape(name);
+        /* Massage the absolute path to a path segment. */
+        /* Copy & paste & search & replace from ls.c */
+        if (ne_path_has_trailing_slash(path)) {
+            current->uri[strlen(path)-1] = '\0';
+        }
 
-	if (strncmp(text, name, len) == 0) {
-	    current = current->next;
-	    /* FIXME: readline docs say readline will free() this when
-	     * it's finished with, although 'memprof' shows that it
-	     * leaks. */
-	    return name;
-	}
+        path = strrchr(path, '/');
+        if (path != NULL && strlen(path+1) > 0) {
+            path++;
+        }
+        else {
+            path = current->uri;
+        }
 
-	current = current->next;
+        native_path = native_path_from_uri(path);
+        if (strncmp(text, native_path, len) == 0) {
+            current = current->next;
+            return native_path;
+        }
+        ne_free(native_path);
+        current = current->next;
     }
-    
+
     return NULL;
 }
 
@@ -816,8 +800,9 @@ char *readline(const char *prompt)
 
 static void init_options(void)
 {
-    char *lockowner, *tmp;
+    char *lockowner;
     char *user = getenv("USER"), *hostname = getenv("HOSTNAME");
+    int utf8_default;
     
     if (user && hostname) {
 	/* set this here so they can override it */
@@ -836,16 +821,31 @@ static void init_options(void)
     lockscope = ne_lockscope_exclusive;
     searchdepth = NE_DEPTH_INFINITE;
 
-    /* This is what Markus Kahn says we should do. */
-    if ((tmp = getenv("LC_ALL")) ||
-	(tmp = getenv("LC_CTYPE")) ||
-	(tmp = getenv("LANG"))) {
-	if (strstr(tmp, "UTF-8")) {
-	    int val = 1;
-	    set_option(opt_utf8, &val);
-	}
-    }
+    /* Detect whether it's possible to directly output UTF-8. */
+#ifdef HAVE_LANGINFO_CODESET
+    out_charset = nl_langinfo(CODESET);
+    utf8_default = strcmp(out_charset, "UTF-8") == 0;
+#else
+    {
+        char *tmp;
 
+        utf8_default = ((tmp = getenv("LC_ALL")) ||
+                        || (tmp = getenv("LC_CTYPE"))
+                        || (tmp = getenv("LANG")))
+            && strstr(tmp, "UTF-8");
+    }
+#endif
+
+#ifndef HAVE_ICONV
+    if (!utf8_default) {
+        fprintf(stderr, _("cadaver: Error: cadaver can only run in a locale "
+                          "using the UTF-8 character encoding since iconv support "
+                          "was not detected at build time.\n"));
+        exit(EXIT_FAILURE);
+    }
+#endif
+
+    set_option(opt_utf8, &utf8_default);
 }
 
 int main(int argc, char *argv[])
